import { messageHistory, recordMessage } from "./messages.js";
import { sendFeedback } from "./feedback.js";
import { generateID } from './id.js';
import { attachLinkTracking } from './link_access.js';
import { disableChatUI } from './ui.js';


const BACKEND_URL = 'https://chat.upb.ro/api/chat/';

const errorMessageText = "Îmi pare rău, nu am putut procesa solicitarea dumneavoastră. Vă rugăm să încercați mai târziu.";
// XXX Bump this number whenever you change the welcome message!
const errorMessageID = "error-message-1";

function enableInputControls() {
    const chatInput = document.getElementById('chat-input');
    chatInput.disabled = false;
    chatInput.placeholder = "Scrieți un mesaj...";

    const chatSubmit = document.getElementById('chat-submit');
    chatSubmit.disabled = false;
    chatSubmit.classList.remove('opacity-50', 'cursor-not-allowed');
}

function disableInputControls() {
    const chatInput = document.getElementById('chat-input');
    chatInput.disabled = true;
    chatInput.placeholder = "Vă rugăm să așteptați răspunsul...";

    const chatSubmit = document.getElementById('chat-submit');
    chatSubmit.disabled = true;
    chatSubmit.classList.add('opacity-50', 'cursor-not-allowed');
}

function removeTypingIndicator() {
    const typingIndicator = document.getElementById('typing-indicator');
    if (typingIndicator) {
        typingIndicator.remove();
    }
}

function renderAssistantMessage(content, isMarkdown = true) {
    const replyElement = document.createElement('div');
    replyElement.className = 'flex mb-3 relative';

    const messageContent = document.createElement('div');
    messageContent.className = 'message-content bg-gray-200 text-black rounded-lg py-2 px-4 max-w-[70%]';

    let html = isMarkdown ? marked.parse(content) : content;
    html = DOMPurify.sanitize(html, { ADD_ATTR: ['target'] });

    const tempElement = document.createElement('div');
    tempElement.innerHTML = html;

    // Ensure links open in new tabs
    tempElement.querySelectorAll('a').forEach(link => {
        link.setAttribute('target', '_blank');
        link.setAttribute('rel', 'noopener noreferrer');
    });

    messageContent.innerHTML = tempElement.innerHTML;
    replyElement.appendChild(messageContent);

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(replyElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;

    return messageContent;
}

async function handlePlainResponse(response, conversationID) {
    try {
        const data = await response.json();
        const assistantMessage = data.reply;
        const messageID = data.messageID;

        const messageContent = renderAssistantMessage('', true); // Initial blank message element

        let html = marked.parse(assistantMessage);
        html = DOMPurify.sanitize(html, { ADD_ATTR: ['target'] });

        const tempElement = document.createElement('div');
        tempElement.innerHTML = html;
        tempElement.querySelectorAll('a').forEach(link => {
            link.setAttribute('target', '_blank');
            link.setAttribute('rel', 'noopener noreferrer');
        });

        messageContent.innerHTML = tempElement.innerHTML;

        removeTypingIndicator();
        const chatMessages = document.getElementById('chat-messages');
        chatMessages.scrollTop = chatMessages.scrollHeight;

        recordMessage({
            conversationID: conversationID,
            messageID: messageID,
            role: "assistant",
            content: assistantMessage
        });

        addFeedbackBar(messageID, messageContent);
        attachLinkTracking(messageID, messageContent);
    } catch (error) {
        console.error('Error processing response:', error);
        // FIXME some better message should be printed here to notify the user
        // of the failure.
        disableChatUI();
    } finally {
        removeTypingIndicator();
        enableInputControls();
    }
}

async function fetchOpenAIResponse(conversationID) {
    try {
        // Prepare the request payload
        const requestOptions = {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({ messages: messageHistory,
                temperature: 0.7
            })
        };

        const response = await fetch(BACKEND_URL, requestOptions);

        if (response.status === 422) {
            removeTypingIndicator();
            let messageText = "Te rog refrazează întrebarea. Nu am înțeles ce dorești să afli."
            // XXX Bump this number whenever you change the welcome message!
            let ambiguousMessageID = "ambiguous-message-1";
            reply(ambiguousMessageID, conversationID, messageText);
        } else if (!response.ok) {
            throw new Error(`API request failed with status ${response.status}`);
        } else {
            return await handlePlainResponse(response, conversationID);
        }

    } catch (error) {
        console.error('Error fetching response:', error);
        removeTypingIndicator();
        reply(conversationID, errorMessageID, errorMessageText);
        disableChatUI();
    }
}

function addFeedbackBar(messageID, messageContent) {
    const feedbackBar = document.createElement('div');
    feedbackBar.className = 'feedback-bar';
    feedbackBar.innerHTML = `
        <div class="feedback-prompt">
            <button class="feedback-button" data-action="LIKE">
                <img src="/res/thumbs-up.svg" alt="Like" />
            </button>
            <button class="feedback-button" data-action="DISLIKE">
                <img src="/res/thumbs-down.svg" alt="Dislike" />
            </button>
        </div>
        <div class="feedback-thanks hidden">
            <span>Mulțumim pentru feedback!</span>
        </div>
    `;

    feedbackBar.querySelectorAll('.feedback-button').forEach(btn => {
        btn.addEventListener('click', () => {
            const action = btn.dataset.action;
            sendFeedback(messageID, action);

            feedbackBar.querySelector('.feedback-prompt').classList.add('hidden');
            feedbackBar.querySelector('.feedback-thanks').classList.remove('hidden');
        });
    });

    messageContent.appendChild(feedbackBar);
}

export function reply(conversationID, messageID, message, isMarkdown = false) {
    let messageContent = renderAssistantMessage(message, isMarkdown);
    recordMessage({
        conversationID: conversationID,
        messageID: messageID,
        role: "assistant",
        content: message
    });
    attachLinkTracking(messageID, messageContent);
}

function showTypingIndicator() {
    const typingElement = document.createElement('div');
    typingElement.id = 'typing-indicator';
    typingElement.className = 'flex mb-3';
    typingElement.innerHTML = `
        <div class="bg-gray-200 text-black rounded-lg py-2 px-4">
          <div class="typing-indicator">
            <span></span>
            <span></span>
            <span></span>
          </div>
        </div>
      `;

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(typingElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;
}

export async function onUserRequest(message, conversationID) {
    const messageElement = document.createElement('div');
    messageElement.className = 'flex justify-end mb-3';
    messageElement.innerHTML = `
        <div class="user-message">
          ${message}
        </div>
        `;

    const chatMessages = document.getElementById('chat-messages');
    chatMessages.appendChild(messageElement);
    chatMessages.scrollTop = chatMessages.scrollHeight;

    recordMessage({
        conversationID: conversationID,
        messageID: generateID(),
        role: "user",
        content: message
    });

    showTypingIndicator();
    disableInputControls();

    try {
        await fetchOpenAIResponse(conversationID);

    } catch (error) {
        console.error('Error fetching response:', error);
        removeTypingIndicator();
        reply(conversationID, errorMessageID, errorMessageText);
    } finally {
        enableInputControls();
    }
}
